window.Gleam.initSearch([{"doc":"xo","title":"xo","content":"# xo\n\nA [Tic-tac-toe](https://en.wikipedia.org/wiki/Tic-tac-toe) library for Gleam.\n\n## An example\n\n```gleam\nimport gleam/io\nimport gleam/string\nimport xo.{X}\n\npub fn main() {\n  let game0 = xo.start(X)\n  let assert Ok(game1) = xo.play(game0, #(1, 1)) // X\n  let assert Ok(game2) = xo.play(game1, #(0, 0)) // O\n\n  let assert Error(xo.OutOfBounds(#(3, 3))) = xo.play(game2, #(3, 3))\n  let assert Error(xo.Occupied(#(1, 1))) = xo.play(game2, #(1, 1))\n\n  let assert Ok(game3) = xo.play(game2, #(2, 2)) // X\n  let assert Ok(game4) = xo.play(game3, #(2, 1)) // O\n\n  io.println(xo.to_string(game4))\n  // =&gt; o...x..ox\n\n  // Where should I play?\n  let moves = xo.get_smart_moves(game4)\n\n  io.println(string.inspect(moves))\n  // =&gt; [#(0, 2), #(1, 2)]\n\n  let assert Ok(game5) = xo.play(game4, #(1, 2)) // X\n\n  // It doesn&#39;t matter where you play now,\n  // you&#39;re going to lose\n  let assert Ok(game6) = xo.play(game5, #(0, 2)) // O\n\n  // Make the winning play\n  let assert Ok(game7) = xo.play(game6, #(1, 0)) // X\n\n  io.println(string.inspect(xo.to_state(game7)))\n  // =&gt; State(X, X, Win(X, [#(#(1, 0), #(1, 1), #(1, 2))]))\n  //          ^  ^  ^\n  //          |  |  |__ outcome\n  //          |  |_____ turn\n  //          |________ first\n}\n```\n\n## The public API at a glance\n\n```txt\nPlayer, next_turn\n\nGame, start\n\nPosition, PlayError, play\n\nRules, default_rules, play_again\n\nState, Outcome, Line, to_state\n\nTile, map\n\nto_string\n\nget_random_move, get_smart_moves\n```\n","url":"index.html"},{"doc":"xo","title":"Game","content":"pub opaque type Game\n\n\nSynonyms:\nxo.Game\nxo Game","url":"xo.html#Game"},{"doc":"xo","title":"Line","content":"pub type Line =\n  #(Position, Position, Position)\n The three positions that make up a winning row, column, or diagonal.\n\nSynonyms:\nxo.Line\nxo Line","url":"xo.html#Line"},{"doc":"xo","title":"Outcome","content":"pub type Outcome {\n  Win(Player, List(Line))\n  Draw(Player)\n  Undecided\n}\n\nWin(Player, List(Line))\n\n\nDraw(Player)\n\n\nUndecided\n\n\nSynonyms:\nxo.Outcome\nxo Outcome","url":"xo.html#Outcome"},{"doc":"xo","title":"PlayError","content":"pub type PlayError {\n  OutOfBounds(Position)\n  Occupied(Position)\n  GameAlreadyEnded\n}\n\nOutOfBounds(Position)\n\n\nOccupied(Position)\n\n\nGameAlreadyEnded\n\n\nSynonyms:\nxo.PlayError\nxo PlayError","url":"xo.html#PlayError"},{"doc":"xo","title":"Player","content":"pub type Player {\n  X\n  O\n}\n\nX\n\n\nO\n\n\nSynonyms:\nxo.Player\nxo Player","url":"xo.html#Player"},{"doc":"xo","title":"Position","content":"pub type Position =\n  #(Int, Int)\n\n\nSynonyms:\nxo.Position\nxo Position","url":"xo.html#Position"},{"doc":"xo","title":"Rules","content":"pub type Rules {\n  Rules(winner_plays_first: Bool, take_turns_on_draw: Bool)\n}\n\nRules(winner_plays_first: Bool, take_turns_on_draw: Bool)\n\n\nSynonyms:\nxo.Rules\nxo Rules","url":"xo.html#Rules"},{"doc":"xo","title":"State","content":"pub type State {\n  State(first: Player, turn: Player, outcome: Outcome)\n}\n\nState(first: Player, turn: Player, outcome: Outcome)\n\n\nSynonyms:\nxo.State\nxo State","url":"xo.html#State"},{"doc":"xo","title":"Tile","content":"pub type Tile =\n  Option(Player)\n\n\nSynonyms:\nxo.Tile\nxo Tile","url":"xo.html#Tile"},{"doc":"xo","title":"default_rules","content":"pub const default_rules: Rules\n The default rules are that the winner plays first and that you must take turns after a draw.\nSynonyms:\nxo.default_rules\nxo default_rules","url":"xo.html#default_rules"},{"doc":"xo","title":"get_random_move","content":"pub fn get_random_move(\n  game: Game,\n  seed: Seed,\n) -&gt; Result(#(#(Int, Int), Seed), Nil)\n\nSynonyms:\nxo.get_random_move\nxo get_random_move","url":"xo.html#get_random_move"},{"doc":"xo","title":"get_smart_moves","content":"pub fn get_smart_moves(game: Game) -&gt; List(#(Int, Int))\n\nSynonyms:\nxo.get_smart_moves\nxo get_smart_moves","url":"xo.html#get_smart_moves"},{"doc":"xo","title":"map","content":"pub fn map(\n  game: Game,\n  f: fn(#(Int, Int), Option(Player)) -&gt; a,\n) -&gt; List(a)\n\nSynonyms:\nxo.map\nxo map","url":"xo.html#map"},{"doc":"xo","title":"next_turn","content":"pub fn next_turn(player: Player) -&gt; Player\n\nSynonyms:\nxo.next_turn\nxo next_turn","url":"xo.html#next_turn"},{"doc":"xo","title":"play","content":"pub fn play(\n  game: Game,\n  pos: #(Int, Int),\n) -&gt; Result(Game, PlayError)\n\nSynonyms:\nxo.play\nxo play","url":"xo.html#play"},{"doc":"xo","title":"play_again","content":"pub fn play_again(game: Game, rules: Rules) -&gt; Game\n Reset the game while respecting the given rules.\nSynonyms:\nxo.play_again\nxo play_again","url":"xo.html#play_again"},{"doc":"xo","title":"start","content":"pub fn start(player: Player) -&gt; Game\n Start a new game such that the given player plays first.\nSynonyms:\nxo.start\nxo start","url":"xo.html#start"},{"doc":"xo","title":"to_state","content":"pub fn to_state(game: Game) -&gt; State\n\nSynonyms:\nxo.to_state\nxo to_state","url":"xo.html#to_state"},{"doc":"xo","title":"to_string","content":"pub fn to_string(game: Game) -&gt; String\n\nSynonyms:\nxo.to_string\nxo to_string","url":"xo.html#to_string"},{"doc":"xo","title":"xo","content":"","url":"xo.html"}]);